//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
#ifndef _connmgrprivapi_h_
#define _connmgrprivapi_h_


#ifndef _PREFAST_
#pragma warning(disable:4068)
#endif

#ifdef _PREFAST_
#pragma prefast(disable:209 307,"Suppress byte count and WCHAR to BSTR cast messages for cellcore")
#endif

#ifdef __cplusplus
extern "C" {
#endif

// Event that indicates whether the BT Phone is being used for an existing data connection
// it indicates to the HFP service that the phone is either free, or that the service
// must signal to the connection manager that it should be closing any existing connection
// to free up the resource
#define BTPHONE_AVAILABLEFORVOICE_EVENT     TEXT("BTPhoneAvailableForVoiceEvent")

// Event that indicates that the HFP service is busy with the phone, i.e. it is attempting
// to dial, or answer, or actually has a live call in progress
#define HFPPHONE_VOICEACTIVE_EVENT          TEXT("HFPPhoneVoiceActiveEvent")

// Event that indicates that there is a phone present (i.e. paired and connected)
#define BTPHONE_PHONEPRESENT_EVENT          TEXT("BTPhonePresentEvent")

// Mutex to make sure we never call Rashangup while we are deactivating DUN profile
#define BTPHONE_DISCONNECTMUTEX             TEXT("BTPhoneDisconnectMutex")

// Max time to wait in either Connect or deactivate during a RasHangup before continuing
#define BTPHONE_MAX_HANGUP_MUTEX_DELAY      ( 6000 )

//---- csppriv.h ex ----------//
// DH - for BT Voice
#define PROVMSG_BTVOICE_SETDATACALLSTATUS       21
#define PROVMSG_BTVOICE_SETDATACALLSTATUSGPRS   22

//---- connprv.h ex ----------//

// DH - Adding additional resource for handling multiple voice devices 
// e.g. embedded phone and BT phone, as well as additional resources
// for controlling simultaneous connections on the external BT phone
// and data connections between the BT phone, and the embedded one

#define CONNMGR_RESOURCE_BTPHONE                0x10000000
#define CONNMGR_RESOURCE_BTDATA                 0x20000000

//---- connmgr.h ex ----------//

// This priority is associated to all voice call connections made over 
// and external phone, such as bt.  This way embedded phone calls will
// trump external phone calls over bt, and we'll know which connections
// to disable.

#define CONNMGR_PRIORITY_VOICE_EX               0x10000

// Used by device management and is the lowest interactive connection
#define CONNMGR_PRIORITY_INTERNALBKGND      0x00010

#ifdef __cplusplus
}
#endif

#endif // _connmgrprivapi_h_
