// i2ctest6Dlg.cpp : implementation file
// Main application file 
// Purpose: This application is designed to write data to an 
// external EEPROM memory module connected to the I2C bus of a
// Texas Instruments EVM3530 development board.
// Solution description: Three types of data are written/read: 
// integer, double and string. Each is written/read to/from a designated 
// edit box. The Write/Read buttons are linked to their paricular edit
// field. 

#include "stdafx.h"
#include "i2ctest6.h"
#include "i2ctest6Dlg.h"

//#define ZONE_ERROR          DEBUGZONE(0)
//#define I2C_DEVICE_NAME         L"I2C1:"
//#include "i2c.h"
#include "i2c.cpp"
#include "oal_i2c.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// Ci2ctest6Dlg dialog

Ci2ctest6Dlg::Ci2ctest6Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(Ci2ctest6Dlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void Ci2ctest6Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EDIT1, m_integer);
	DDX_Control(pDX, IDC_EDIT2, m_double);
	DDX_Control(pDX, IDC_EDIT3, m_string);
}

BEGIN_MESSAGE_MAP(Ci2ctest6Dlg, CDialog)
#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
	ON_WM_SIZE()
#endif
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON1, &Ci2ctest6Dlg::OnBnClickedButton1)
	ON_BN_CLICKED(IDC_BUTTON3, &Ci2ctest6Dlg::OnBnClickedButton3)
	ON_BN_CLICKED(IDC_BUTTON2, &Ci2ctest6Dlg::OnBnClickedButton2)
	ON_BN_CLICKED(IDC_BUTTON4, &Ci2ctest6Dlg::OnBnClickedButton4)
	ON_BN_CLICKED(IDC_BUTTON5, &Ci2ctest6Dlg::OnBnClickedButton5)
	ON_BN_CLICKED(IDC_BUTTON6, &Ci2ctest6Dlg::OnBnClickedButton6)
	ON_BN_CLICKED(IDC_BUTTON7, &Ci2ctest6Dlg::OnBnClickedButton7)
END_MESSAGE_MAP()


// Ci2ctest6Dlg message handlers

BOOL Ci2ctest6Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
void Ci2ctest6Dlg::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/)
{
	if (AfxIsDRAEnabled())
	{
		DRA::RelayoutDialog(
			AfxGetResourceHandle(), 
			this->m_hWnd, 
			DRA::GetDisplayMode() != DRA::Portrait ? 
			MAKEINTRESOURCE(IDD_I2CTEST6_DIALOG_WIDE) : 
			MAKEINTRESOURCE(IDD_I2CTEST6_DIALOG));
	}
}
#endif

HANDLE hPort = NULL ; 	// handle of the open i2c bus
BOOL result;		// variable used for testing operation success
DWORD dwSlaveAddr = 0x50;	// the address of the EEPROM memory module
#define IOCTL_I2C_SET_SLAVE_ADDRESS   0x220800 	//set slave address command
 //unsigned char outdata[2] = {0,3};
 unsigned char outdata[6] = {54,11,43,76,32,1};	///test vectors
 unsigned char indata[6] = {0,0,0,0,0,0};		///
 //unsigned char indata[2] = {0,0};
 DWORD len = 6;		// number of successfuly written bytes 
 TCHAR szText[64];	/// variable used to display the address to which the
				/// write operation was executed

  typedef union //declaration for the union used in the integer write operation
 {
 int valoarei; // value read from edit box
 unsigned char buffi[4]; // value read from the box split into 4 bytes
 }memint;

 typedef union //declaration for the union used in the double write operation
 {
 double valoared; // value read from edit box
 unsigned char buffd[8]; // value read from the box split into 8 bytes
 }memdoub;


void Ci2ctest6Dlg::OnBnClickedButton1()
{
	// Test Button
	// The test sequence consists of consecutive write and read operations 
	// to every 7 bit address on the given I2C bus

  // open a handle to I2C2 bus for read / write operations 
  hPort = CreateFile(TEXT("I2C2:"),GENERIC_READ | GENERIC_WRITE,0,0,OPEN_EXISTING,0,NULL);
  
  if( hPort == INVALID_HANDLE_VALUE) // if FAIL display error message and close handle
  {
  AfxMessageBox(L"unable open i2c2 port", NULL, NULL);
  CloseHandle(hPort);
//  return 0;
 }
  // scan every 7 bit address
  for (dwSlaveAddr = 0x00; dwSlaveAddr < 128; dwSlaveAddr++)
  {
  // set slave address of i2c device
  result = DeviceIoControl(hPort, IOCTL_I2C_SET_SLAVE_ADDRESS, &dwSlaveAddr,                             
     sizeof(dwSlaveAddr), NULL, 0, NULL, NULL);
// if FAIL display error message and close handle  
if (!result) {
   AfxMessageBox(L"DeviceIoControl failed", NULL, NULL);
   CloseHandle(hPort);
   }
    // internal address of the peripheral 
    SetFilePointer(hPort, 0, NULL, FILE_BEGIN);
// send data to current address
   result = WriteFile(hPort, outdata, 2, &len, 0);
// if SUCCESS display message
if (result) {
    wsprintf(szText, TEXT("address %d"), dwSlaveAddr);
    AfxMessageBox(szText,NULL,NULL);
    }

    SetFilePointer(hPort,0, NULL, FILE_BEGIN);
// ask for data from current address   
    result = ReadFile(hPort, indata, 6, &len, 0);
// if SUCCESS display message
    if (result) {
     AfxMessageBox(L"Test Success", NULL, NULL);
    }
  } // A valid address is found when both write and read operations are successful
	CloseHandle(hPort);
}


void Ci2ctest6Dlg::OnBnClickedButton3()
{
	//Integer Read Button 

	CString integerr;	// string to be placed in the edit box
	unsigned int buff[1];	// read buffer variable
	// open a handle to I2C2 bus for read / write operations 
	hPort = CreateFile(TEXT("I2C2:"),GENERIC_READ | GENERIC_WRITE,0,0,OPEN_EXISTING,0,NULL);
	if( hPort == INVALID_HANDLE_VALUE) 
	{
	AfxMessageBox(L"unable open i2c2 port", NULL, NULL);
	CloseHandle(hPort);
	}
	// set slave address of i2c device
	result = DeviceIoControl(hPort, IOCTL_I2C_SET_SLAVE_ADDRESS, &dwSlaveAddr, 
		sizeof(dwSlaveAddr), NULL, 0, NULL, NULL);
	if (!result) {
	AfxMessageBox(L"DeviceIoControl failed", NULL, NULL);
	CloseHandle(hPort);
	}
	// internal address of the peripheral from where the integer will be read
	SetFilePointer(hPort, 0, NULL, FILE_BEGIN);
	// read 4 bytes to buff 
	result = ReadFile(hPort, buff, 4, &len, 0);
	if (result) {
	AfxMessageBox(L"Read Success", NULL, NULL);
	}
	CloseHandle(hPort);
	// read data formatting for edit box display
	integerr.Format(L"%u",buff[0]);
	m_integer.SetWindowTextW(integerr);
}


void Ci2ctest6Dlg::OnBnClickedButton2()
{
	//Integer Write Button
	CString integerw;	// string read from the edit box
	memint x;	// union used for data write
	// read data formatting for memory write
	m_integer.GetWindowTextW(integerw);
	x.valoarei = wcstoul(integerw.GetBuffer(),'\0',10);
	// open a handle to I2C2 bus for read / write operations 
	hPort = CreateFile(TEXT("I2C2:"),GENERIC_READ | GENERIC_WRITE,0,0,OPEN_EXISTING,0,NULL);
	if( hPort == INVALID_HANDLE_VALUE) 
	{
	AfxMessageBox(L"unable open i2c2 port", NULL, NULL);
	CloseHandle(hPort);
	}
	// set slave address of i2c device  
	result = DeviceIoControl(hPort, IOCTL_I2C_SET_SLAVE_ADDRESS, &dwSlaveAddr, 
		sizeof(dwSlaveAddr), NULL, 0, NULL, NULL);
	if (!result) {
	AfxMessageBox(L"DeviceIoControl failed", NULL, NULL);
	CloseHandle(hPort);
	}
	// internal address of the peripheral where the integer will be written
	SetFilePointer(hPort, 0, NULL, FILE_BEGIN);
	// write buffi to memory; it holds 4 bytes
	result = WriteFile(hPort, x.buffi, 4, &len, 0);
	CloseHandle(hPort);
	if (result) {
	wsprintf(szText, TEXT("address %d"), dwSlaveAddr);
	AfxMessageBox(szText,NULL,NULL);
	}
	//CloseHandle(hPort);
}

void Ci2ctest6Dlg::OnBnClickedButton4()
{
	//Double Write Button
	CString doubw;	// string read from the edit box
	memdoub x;	// union used for data write
	// read data formatting for memory write
	m_double.GetWindowTextW(doubw);
	x.valoared = wcstod(doubw.GetBuffer(),NULL);
	// open a handle to I2C2 bus for read / write operations 
	hPort = CreateFile(TEXT("I2C2:"),GENERIC_READ | GENERIC_WRITE,0,0,OPEN_EXISTING,0,NULL);
	if( hPort == INVALID_HANDLE_VALUE) 
	{
	AfxMessageBox(L"unable open i2c2 port", NULL, NULL);
	CloseHandle(hPort);
	}
  	// set slave address of i2c device  
	result = DeviceIoControl(hPort, IOCTL_I2C_SET_SLAVE_ADDRESS, &dwSlaveAddr,                             
		sizeof(dwSlaveAddr), NULL, 0, NULL, NULL);
	if (!result) {
	AfxMessageBox(L"DeviceIoControl failed", NULL, NULL);
	CloseHandle(hPort);
	}
	// internal address of the peripheral where the double will be written
	// had major problems with the 2nd parameter; tried 2, 5, 10 and still got data overlap
	// 1000 works but could be overkill
	SetFilePointer(hPort, 1000, NULL, FILE_BEGIN);
	// communication with the memory module fails sometimes
	// this loop tries to write the given value until the operation is successful
	// maximum of 20 tries
	for (int i=0 ;i<20;i++)
	{
	// write buffd to memory; it holds 8 bytes
	result = WriteFile(hPort, x.buffd, 8, &len, 0);
	if (result)	/// if operation was successful 
		i=20;	/// exit FOR loop
	}
	if (result) {
	wsprintf(szText, TEXT("address %d"), dwSlaveAddr);
	AfxMessageBox(szText,NULL,NULL);
	}
	CloseHandle(hPort);
}

void Ci2ctest6Dlg::OnBnClickedButton5()
{
	// Double Read Button
	CString doubr;	// string to be placed in the edit box
	double buff[1];	// read buffer variable
	// open a handle to I2C2 bus for read / write operations 
	hPort = CreateFile(TEXT("I2C2:"),GENERIC_READ | GENERIC_WRITE,0,0,OPEN_EXISTING,0,NULL);
	if( hPort == INVALID_HANDLE_VALUE) 
	{
	AfxMessageBox(L"unable open i2c2 port", NULL, NULL);
	CloseHandle(hPort);
	}
  	// set slave address of i2c device  
	result = DeviceIoControl(hPort, IOCTL_I2C_SET_SLAVE_ADDRESS, &dwSlaveAddr,                             
		sizeof(dwSlaveAddr), NULL, 0, NULL, NULL);
	if (!result) {
	AfxMessageBox(L"DeviceIoControl failed", NULL, NULL);
	CloseHandle(hPort);
	}
	// internal address of the peripheral from where the double will be read
	SetFilePointer(hPort, 1000 , NULL, FILE_BEGIN);
	// read 8 bytes to buff 
	result = ReadFile(hPort, buff, 8, &len, 0);
	if (result) {
	AfxMessageBox(L"Read Success", NULL, NULL);
	}
	CloseHandle(hPort);
	// read data formatting for edit box display	
	doubr.Format(L"%g",buff[0]);
	m_double.SetWindowTextW(doubr);
}

void Ci2ctest6Dlg::OnBnClickedButton6()
{
	// String Write Button
	CString strw;	// string read from the edit box
	CString strb;	// formatted string
	char strwr[10];	// byte array to be written
	// read string from text box, format it 
	m_string.GetWindowTextW(strw);
	strb.Format(L"%s",strw);
	wcstombs(strwr,strb.GetBuffer(),10);	//conversion from wchar to char/byte 
	// open a handle to I2C2 bus for read / write operations 
	hPort = CreateFile(TEXT("I2C2:"),GENERIC_READ | GENERIC_WRITE,0,0,OPEN_EXISTING,0,NULL);
	if( hPort == INVALID_HANDLE_VALUE) 
	{
	AfxMessageBox(L"unable open i2c2 port", NULL, NULL);
	CloseHandle(hPort);
	}
	// set slave address of i2c device  
	result = DeviceIoControl(hPort, IOCTL_I2C_SET_SLAVE_ADDRESS, &dwSlaveAddr,                             
		sizeof(dwSlaveAddr), NULL, 0, NULL, NULL);
	if (!result) {
	AfxMessageBox(L"DeviceIoControl failed", NULL, NULL);
	CloseHandle(hPort);
	}
	// internal address of the peripheral where the string will be written
	SetFilePointer(hPort, 2000, NULL, FILE_BEGIN);
	// write buffer to memory; it holds 10 bytes maximum
	result = WriteFile(hPort, strwr, (lstrlen(strb.GetBuffer()))+1, &len, 0);
	if (result) {
	wsprintf(szText, TEXT("address %d"), dwSlaveAddr);
	AfxMessageBox(szText,NULL,NULL);
	}
	CloseHandle(hPort);

}

void Ci2ctest6Dlg::OnBnClickedButton7()
{
	// String Read Button
	CString strr;	// string to be placed in the edit box
	char strrd[10];	// read buffer string
	char strbu[20];	// read string converted to wchar
	// open a handle to I2C2 bus for read / write operations 
	hPort = CreateFile(TEXT("I2C2:"),GENERIC_READ | GENERIC_WRITE,0,0,OPEN_EXISTING,0,NULL);
	if( hPort == INVALID_HANDLE_VALUE) 
	{
	AfxMessageBox(L"unable open i2c2 port", NULL, NULL);
	CloseHandle(hPort);
	}
	// set slave address of i2c device
	result = DeviceIoControl(hPort, IOCTL_I2C_SET_SLAVE_ADDRESS, &dwSlaveAddr,                             
		sizeof(dwSlaveAddr), NULL, 0, NULL, NULL);
	if (!result) {
	AfxMessageBox(L"DeviceIoControl failed", NULL, NULL);
	CloseHandle(hPort);
	}
	SetFilePointer(hPort, 2000 , NULL, FILE_BEGIN);
	// read 10 bytes to the given buffer
    result = ReadFile(hPort, strrd, 10, &len, 0);
    if (result) {
	AfxMessageBox(L"Read Success", NULL, NULL);
    }
	CloseHandle(hPort);
	//mbstowcs(straf,strrd,10); // standard conversion function
	for(int i=0; i<len;i++)		/// char to wchar 
		strbu[i*2]=strrd[i];	/// conversion routine
	// read data formatting for edit box display
	strr.Format(L"%s",strbu);
	m_string.SetWindowTextW(strr.GetBuffer());

}
