//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
//

#ifndef __LF_PLUGIN_H
#define __LF_PLUGIN_H

// Location Framework Plugin reference.  Plugins must implement functions
// declared here to support provider and/or resolver functionality.

#include <LFApi.h>

#ifdef __cplusplus
extern "C" {
#endif

// Flags that are specific to all plugins
#define PLUGIN_FLAGS_DISABLED              0x00000001

//
// Provider specific structures and functions
//

typedef struct _PROVIDER_CONTROL_BLOCK {
    DWORD version;
    HANDLE provContext;
    PROVIDER_INFORMATION *pProvInfo;
    DWORD (WINAPI *NewProviderReport)(HANDLE provContext, LOCATION_REPORT *pNewReport);
    DWORD (WINAPI *ProviderUnavailable)(HANDLE provContext);
} PROVIDER_CONTROL_BLOCK, *PPROVIDER_CONTROL_BLOCK;


DWORD  ProviderInitialize(PROVIDER_INFORMATION *pProvInfo);
DWORD  ProviderGetLocation(PROVIDER_CONTROL_BLOCK *pProvControlBlock);
DWORD  ProviderStop(void);
DWORD  ProviderUnInitialize(void);

HANDLE ProviderIoctlOpen(void);
DWORD  ProviderIoctlCall(HANDLE hProv, DWORD dwCode, BYTE *pBufIn, DWORD cbIn, BYTE *pBufOut, DWORD *pcbOut);
DWORD  ProviderIoctlClose(HANDLE hProv);

//
// Resolver specific structures and functions
//

typedef struct _RESOLVER_CONTROL_BLOCK {
    DWORD version;
    HANDLE resContext;
    RESOLVER_INFORMATION *pResInfo;

    LOCATION_REPORT *provReport;
    GUID  typesToResolve[MAX_PLUGIN_REPORT_TYPES];
    DWORD numTypesToResolve;

    DWORD (WINAPI *NewResolverReport)(HANDLE resContext, LOCATION_REPORT *pNewReport);
} RESOLVER_CONTROL_BLOCK, *PRESOLVER_CONTROL_BLOCK;


DWORD ResolverInitialize(RESOLVER_INFORMATION *pResInfo);
DWORD ResolverGetLocation(RESOLVER_CONTROL_BLOCK *pResControlBlock);
DWORD ResolverStop();
DWORD ResolverUnInitialize(void);

HANDLE ResolverIoctlOpen(void);
DWORD  ResolverIoctlCall(HANDLE h, DWORD dwCode, BYTE *pBufIn, DWORD cbIn, BYTE *pBufOut, DWORD *pcbOut);
DWORD  ResolverIoctlClose(HANDLE h);

#ifdef __cplusplus
}
#endif

#endif
